package com.persagy.iot.app

import com.persagy.iot.bean.IOTData
import com.persagy.iot.func.{IOTSinkFunction, SplitData}
import com.persagy.iot.utils.KafkaUtil
import org.apache.flink.api.common.serialization.SimpleStringSchema
import org.apache.flink.streaming.api.TimeCharacteristic
import org.apache.flink.streaming.api.functions.timestamps.BoundedOutOfOrdernessTimestampExtractor
import org.apache.flink.streaming.api.scala._
import org.apache.flink.streaming.api.windowing.time.Time
import org.apache.flink.streaming.connectors.kafka.FlinkKafkaConsumer

object IOTApp {

  /** 只要状态为 report 的数据 */
  val common_type: String = "report"

  /** 判断功能号的值是否存在 */
  val no_value: String = "ff ff ff ff"

  def main(args: Array[String]): Unit = {
    val env: StreamExecutionEnvironment = StreamExecutionEnvironment.getExecutionEnvironment

    /* 设置并行度 */
    env.setParallelism(1)

    /* 设置时间语义 */
    env.setStreamTimeCharacteristic(TimeCharacteristic.EventTime)

    val inputStream: DataStream[String] =
      env.addSource( new FlinkKafkaConsumer[String](KafkaUtil.topic, new SimpleStringSchema(), KafkaUtil.kafkaConsumerProperties()) )

    /* 过滤掉非 report 的数据 */
    val filterStream: DataStream[String] = inputStream.filter(data => {

      val arr1: Array[String] = data.split("\t")
      if (arr1.length != 4){
        false
      } else {
        val arr2: Array[String] = arr1(3).split(";")
        common_type.equals(arr2(2))
      }
    })

    /* 转化为实体类流 */
    val iotFilterDataStream: DataStream[IOTData] = filterStream.flatMap(new SplitData)
//    iotFilterDataStream.print("iotFilterDataStream")

//    inputStream.print("inputStream")

    val waterStream: DataStream[IOTData] = iotFilterDataStream
      /* 设置水位时间 */
      .assignTimestampsAndWatermarks(new BoundedOutOfOrdernessTimestampExtractor[IOTData](Time.minutes(30)) {
        override def extractTimestamp(element: IOTData) = {
          element.eventTime - 1
        }
      })

    /* 定义侧输出流 */
    val sideOutputTag = new OutputTag[IOTData]("late data")

    val windowsStream: DataStream[IOTData] = waterStream.keyBy(_.id)
      .timeWindow(Time.minutes(15))
      /* 允许处理数据的最迟时间 */
      .allowedLateness(Time.minutes(60))
      /* 侧输出流 */
      .sideOutputLateData(sideOutputTag)
      .aggregate(new IOTAgg(), new IOTWindowResult())

    /* 设置要选取的事件时间 */
//    val assignStream: DataStream[IOTData] = iotDataStream.assignAscendingTimestamps(_.eventTime)
    windowsStream.addSink(new IOTSinkFunction)
//    windowsStream.getSideOutput(sideOutputTag).addSink(new IOTSourceFunction)

    env.execute("iot data collect")
  }


}

